/*
 * Copyright (c) 2005 SilverStorm Technologies.  All rights reserved.
 *
 * This software is available to you under the OpenIB.org BSD license
 * below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: ipoib_shutter.h 1611 2006-08-20 14:48:55Z sleybo $
 */



#ifndef _IPOIB_SHUTTER_H_
#define _IPOIB_SHUTTER_H_




/*
 * Define the max numbers of operations that can be simultaniously done
 */
#define MAX_OPERATIONS	0x10000000

typedef struct _shutter_t {
	long opertaions;

}	shutter_t;

static inline void shutter_init(shutter_t* p_shutter)
{
	p_shutter->opertaions = 0;
}

static inline int shutter_use(shutter_t * p_shutter)
{
	long res;
	res = InterlockedExchangeAdd(&p_shutter->opertaions, 1); 
	ASSERT(res < MAX_OPERATIONS);
	if (res >= 0 ) {
		// Everything went OK, we can do the operation
		return res;
	}
	// The object was already shutdown
	InterlockedExchangeAdd(&p_shutter->opertaions, -1); 
	return -1;
}

static inline void shutter_loose(shutter_t * p_shutter, long count)
{
	InterlockedExchangeAdd(&p_shutter->opertaions, -count); 
}

static inline void shutter_shut(shutter_t * p_shutter)
{
	// Mark the counter as locked
	long res;
	LARGE_INTEGER	interval;
	res = InterlockedExchangeAdd(&p_shutter->opertaions, -MAX_OPERATIONS);
	if (res == 0) 
	{
		// The object had no user, quit
		return;
	}
	if (res <= -MAX_OPERATIONS)
	{
		// This probably means that we have another bug, but I want to give this
		// a chance (someone has already started to shut) ?????????
		InterlockedExchangeAdd(&p_shutter->opertaions, MAX_OPERATIONS);
	}
	// We now wait for the object to reach to -MAX_OPERATIONS
	interval.QuadPart = 0;
	for(;;) {
		res = InterlockedExchangeAdd(&p_shutter->opertaions, 0);
		if (res == -MAX_OPERATIONS) {
			return;
		}
		KeDelayExecutionThread( KernelMode, FALSE, &interval );
	}
}

#endif _IPOIB_SHUTTER_H_
